import os
from .utils import download, check_dir


def fetch(corpus_name, root_dir):
    """
    Examples::
        >>> from Korpora.fetch import fetch
        >>> fetch('nsmc', './Korpora')
    """
    if corpus_name not in DATA_LOCATIONS:
        raise ValueError(f'support only {set(DATA_LOCATIONS.keys())}')
    locations = DATA_LOCATIONS[corpus_name]
    print(f'Fetch {corpus_name} to {os.path.abspath(root_dir)}')
    for information in locations:
        if information['method'] == 'download':
            do_download(information, root_dir)


def do_download(information, root_dir):
    url = information['url']
    destination = os.path.join(root_dir, information['destination'])
    check_dir(destination)
    corpus_name = information['destination'].split('/')[0]
    download(url, destination, corpus_name)


DATA_LOCATIONS = {
    'nsmc': [
        {
            'url': 'https://raw.githubusercontent.com/e9t/nsmc/master/ratings_train.txt',
            'destination': 'nsmc/ratings_train.txt',
            'method': 'download'
        },
        {
            'url': 'https://raw.githubusercontent.com/e9t/nsmc/master/ratings_test.txt',
            'destination': 'nsmc/ratings_test.txt',
            'method': 'download'
        },
    ],
}


from dataclasses import dataclass
from typing import List

from .utils import default_korpora_path


@dataclass
class KorpusData:
    texts: List[str]

    def __len__(self):
        return len(self.texts)


class Korpus:
    def cleaning(self, raw_documents: List[str], **kargs):
        """`raw_data` to `sentences`"""
        raise NotImplementedError('Implement this function')

    def get_all_texts(self):
        raise NotImplementedError('Implement this function')

    def save(self, root_dir):
        """save prorce` to `sentences`"""
        raise NotImplementedError('Implement this function')



from .korpora_nsmc import NSMC
from .utils import default_korpora_path


class Korpora:
    """
    Examples::
        >>> from Korpora import Korpora
        >>> nsmc = Korpora.load('nsmc')
        >>> len(nsmc.train.texts)   # 150000
        >>> len(nsmc.train.labels)  # 50000
    """
    @classmethod
    def load(cls, corpus_names, root_dir=None, force_download=False):
        return_single = isinstance(corpus_names, str)
        if return_single:
            corpus_names = [corpus_names]

        if root_dir is None:
            root_dir = default_korpora_path

        corpora = [KORPORA[corpus_name](root_dir, force_download) for corpus_name in corpus_names]
        if return_single:
            return corpora[0]
        return corpora


KORPORA = {
    'nsmc': NSMC
}



import os
from os.path import expanduser
from tqdm import tqdm
from urllib import request


default_korpora_path = f'{expanduser("~")}/Korpora/'


def check_path(path):
    return os.path.exists(path)


def check_dir(filepath):
    dirname = os.path.abspath(os.path.dirname(filepath))
    if not os.path.exists(dirname):
        os.makedirs(dirname)


def load_text(path, num_heads=0):
    with open(path, encoding='utf-8') as f:
        lines = [line.rstrip('\n') for line in f]
    if num_heads > 0:
        lines = lines[num_heads:]
    return lines


def _reporthook(t):
    """ ``reporthook`` to use with ``urllib.request`` that prints the process of the download.

    Uses ``tqdm`` for progress bar.

    **Reference:**
    https://github.com/tqdm/tqdm

    Args:
        t (tqdm.tqdm) Progress bar.

    Example:
        >>> with tqdm(unit='B', unit_scale=True, miniters=1, desc=filename) as t:  # doctest: +SKIP
        ...   urllib.request.urlretrieve(file_url, filename=full_path, reporthook=reporthook(t))
    """
    last_b = [0]

    def inner(b=1, bsize=1, tsize=None):
        """
        Args:
            b (int, optional): Number of blocks just transferred [default: 1].
            bsize (int, optional): Size of each block (in tqdm units) [default: 1].
            tsize (int, optional): Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            t.total = tsize
        t.update((b - last_b[0]) * bsize)
        last_b[0] = b

    return inner

def download(url, local_path, corpus_name=''):
    filename = os.path.basename(local_path)
    with tqdm(unit='B', unit_scale=True, miniters=1, desc=f'[{corpus_name}] download {filename}') as t:
        request.urlretrieve(url, filename=local_path, reporthook=_reporthook(t))



import os
from typing import List

from .korpora import Korpus, KorpusData
from .fetch import fetch
from .utils import check_path, load_text


class NSMCData(KorpusData):
    labels: List[str]

    def __init__(self, texts, labels):
        if len(texts) != len(labels):
            raise ValueError('`texts` and `labels` must be same length')
        super().__init__(texts)
        self.labels = labels


class NSMC(Korpus):
    """Reference: https://github.com/e9t/nsmc

    Naver sentiment movie corpus v1.0
    This is a movie review dataset in the Korean language.
    Reviews were scraped from Naver Movies.

    The dataset construction is based on the method noted in
    [Large movie review dataset][^1] from Maas et al., 2011.

    [^1]: http://ai.stanford.edu/~amaas/data/sentiment/
    """
    def __init__(self, root_dir, force_download=False):
        train_path = os.path.join(root_dir, 'nsmc/ratings_train.txt')
        test_path = os.path.join(root_dir, 'nsmc/ratings_test.txt')
        if (force_download or
            not check_path(train_path) or
            not check_path(test_path)
           ):
            fetch('nsmc', root_dir)

        train_texts, train_labels = self.cleaning(load_text(train_path, num_heads=1))
        test_texts, test_labels = self.cleaning(load_text(test_path, num_heads=1))
        self.train = NSMCData(train_texts, train_labels)
        self.test = NSMCData(test_texts, test_labels)

    def cleaning(self, raw_lines: List[str]):
        separated_lines = [line.split('\t') for line in raw_lines]
        for i_sent, separated_line in enumerate(separated_lines):
            if len(separated_line) != 3:
                raise ValueError(f'Found some errors in line {i_sent}: {separated_line}')
        _, texts, labels = zip(*separated_lines)
        labels = [int(label) for label in labels]
        return texts, labels

    def get_all_texts(self):
        return self.train.texts + self.test.texts

    def get_all_labels(self):
        return self.train.labels + self.test.labels
